from lib2d.gamestate import GameState
from lib2d.tilemap import BigMap, OneTileMap, View, ThreadedView
from lib2d.renderer import Camera
from lib2d.dialog import TextDialog 
from lib2d.buttons import *
from lib2d.statedriver import driver as sd
from lib2d import param, gfx, sfx

import os.path



class IntroState(GameState):
    """
    This state introduces the player to the story of the game
    """

    def __init__(self):
        GameState.__init__(self)

        # cloud parallax layer
        #cloudlayer = OneTileMap("clouds.png")
        #self.view1 = View(cloudlayer, gfx.screen_dim /2)

        self.dialogs = []

        with open(os.path.join(param.resource_path, "scripts", "intro.txt")) as fh:
            for line in fh:
                line = line.strip()
                if line != "":
                    self.dialogs.append(line)

        self.dialogs.reverse()

        self.show_image = None

    def deactivate(self):
        GameState.deactivate(self)
        sfx.fadeout_music()

    def activate(self):
        GameState.activate(self)
        sfx.music("relent.ogg", loops=-1)
        self.cleared = False

    def reactivate(self):
        GameState.reactivate(self)
        self.cleared = False

    def draw(self, surface):
        if not self.cleared:
            surface.fill((0,0,0))
            self.cleared = True

            if self.show_image != None:
                surface.blit(self.show_image[0], self.show_image[1])

    def update(self, time):

        try:
            text = self.dialogs.pop()
        except IndexError:
            sd.done()
            return

        if text[:6] == "#image":
            fill = False
            tag, path = text.split(":")

            if path[0] == "+":
                fill = True
                path = path[1:]

            if tag[-8:] == "colorkey":
                image = gfx.load_colorkey(path)
                print "key"
            elif tag[-5:] == "alpha":
                image = gfx.load_image_with_alpha(path)
            else:
                image = gfx.load(path)
 
            rect = image.get_rect()
            if fill:
                rect.topleft = ((sd.get_size()[0] / 2) - (rect.width / 2), 0)
            else:
                rect.topleft = ((sd.get_size()[0] / 2) - (rect.width / 2), 10)
            
            self.show_image = (image, rect)
            self.cleared = False

        else:
            sd.start(TextDialog(text))

    def handle_commandlist(self, cmdlist):
        pass
