"""
Copyright 2010, 2011  Leif Theden


This file is part of lib2d.

lib2d is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

from gamestate import GameState
from dialog import TextDialog 
from buttons import *
from statedriver import driver as sd
import param, gfx, sfx

import os.path



class Cutscene(GameState):
    """
    This state can read a text file and make a series of dialogs from it.

    Each individual dialog must be it's own line.
    Blank lines are ignored.
    Comments may be added using a #.  Inline comments are not supported.

    Special commands are available:

    #image:filename             load the image and blit it
    #image-colorkey:filename    load image and use top-left corner as colorkey
    #image-alpha:filename       load image with per-pixel alpha

    * if you add a "+" before the filename, the image will fit the screen (kinda)

    """

    def __init__(self, path):
        GameState.__init__(self)

        self.dialogs = []

        with open(path) as fh:
            for line in fh:
                line = line.strip()
                if line != "":
                    self.dialogs.append(line)

        self.dialogs.reverse()

        self.queue_image = None
        self.queue_music = None

    def deactivate(self):
        GameState.deactivate(self)
        sfx.fadeout_music()

    def activate(self):
        GameState.activate(self)

        # this is hack, for sure, but ensure our music will start
        # playing if we are being transition'd from another state

        if self.dialogs[-1][:6] == "#music":
            text = self.dialogs.pop()
            tag, path = text.split(":")
            sfx.music(path, loops=-1)
        
        self.cleared = False

    def reactivate(self):
        GameState.reactivate(self)
        self.cleared = False

    def draw(self, surface):
        if not self.cleared:
            surface.fill((0,0,0))
            self.cleared = True

            if self.queue_image != None:
                surface.blit(self.queue_image[0], self.queue_image[1])

    def update(self, time):
        try:
            text = self.dialogs.pop()
        except IndexError:
            sd.done()
            return

        if text[:6] == "#image":
            fill = False
            tag, path = text.split(":")

            if path[0] == "+":
                fill = True
                path = path[1:]

            if tag[-8:] == "colorkey":
                image = gfx.load_colorkey(path)
                print "key"
            elif tag[-5:] == "alpha":
                image = gfx.load_image_with_alpha(path)
            else:
                image = gfx.load(path)
 
            rect = image.get_rect()
            if fill:
                rect.topleft = ((sd.get_size()[0] / 2) - (rect.width / 2), 0)
            else:
                rect.topleft = ((sd.get_size()[0] / 2) - (rect.width / 2), 10)
            
            self.queue_image = (image, rect)
            self.cleared = False

        elif text[:6] == "#music":
            tag, path = text.split(":")
            self.queue_music = path

        else:
            sd.start(TextDialog(text))

    def handle_event(self, event):
        pass

    def handle_commandlist(self, cmdlist):
        pass
