"""
Copyright 2010, 2011  Leif Theden


This file is part of lib2d.

lib2d is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

class ObjectHandler(object):
    """
    these classes can handle other types of objects
    """

    def _add_other(self, obj):
        raise ValueError, "This type of object is not handled: %s" % obj.__class__

    def add(self, obj):
        # return True if this class will accept this type of object
        raise NotImplementedError

    def remove(self, obj):
        raise NotImplementedError

class UpdateableObject(object):
    def __init__(self):
        self._is_enabled = True
        self.init()

    # just a hook for subclasses
    def init(self):
        pass

    def update(self, time):
        pass

    def enable(self):
        self._is_enabled = True

    def disable(self):
        self._is_enabled = False

class DrawableObject(object):
    def __init__(self):
        super(DrawableObject, self).__init__()
        self._is_visible = True
        self.dirty = False

    def draw(self, surface):
        raise NotImplementedError, self

    def show(self):
        self.is_visible = True

    def hide(self):
        self._is_visible = False

class KeyHandler(object):
    def key_down(self, key):
        pass

    def key_up(self, key):
        pass 

class EventHandler(object):
    """
    Events from PyGame with trickle down through listeners
    based on order of insertion.
    Make sure to return the event if you want to all lesser
    Listeners to use it as well.  Return None to kill it.
    """

    def handle(self, event):
        return event

