"""
Copyright 2010, 2011  Leif Theden


This file is part of lib2d.

lib2d is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

import param

import pygame, os.path, sys

pygame.mixer.init()

DEBUG = False

def debug(text):
    if DEBUG: sys.stdout.write(text)


sound_cache = {}

class NoSound:
    def play(self): pass
    def stop(self): pass

dummy = NoSound()

def load(name):
    global sound_cache

    if not pygame.mixer:
        debug("Cannot load sound: pygame.mixer not ready\n")
        return dummy

    fullpath = os.path.join(param.resource_path, "sounds", name)

    # try to the the cach first
    try:
        return sound_cache[fullpath]
    except KeyError:
        pass

    # pygame will load a sound even if it doesn't exist... do a sanity check.
    open(fullpath).close()

    try:
        sound = pygame.mixer.Sound(fullpath)
    except pygame.error, message:
        debug("Cannot load sound: %s\n" % fullpath)
        debug("%s\n" % message)
        return dummy

    # update the cache
    sound_cache[fullpath] = sound

    return sound

def music(name, *args, **kwargs):
    stop_music()

    fullpath = os.path.join(param.resource_path, "music", name)

    pygame.mixer.music.load(fullpath)

    if "loops" in kwargs.keys():
        pygame.mixer.music.play(kwargs["loops"])

def stop_music():
    pygame.mixer.music.stop()

def fadeout_music(time=3000):
    pygame.mixer.music.fadeout(time)
